#  @copyright Steve Keen 2012
#  @author Russell Standish
#  This file is part of Minsky.
#
#  Minsky is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Minsky is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Minsky.  If not, see <http://www.gnu.org/licenses/>.
#

# Godley table


set currGodley -1

proc createGodleyWindow {} {
    toplevel .godley 
    wm title .godley "Godley Table"
    wm withdraw .godley

    set t .godley.table
    table $t \
        -rows 1 \
        -cols 1 \
        -colwidth 20 \
        -titlerows 0 \
        -titlecols 0 \
        -yscrollcommand { .godley.sy set } \
        -xscrollcommand { .godley.sx set } \
        -coltagcommand colorize \
        -flashmode off \
        -selectmode extended \
        -ellipsis on \
        -width 20 -height 20 \
        -colstretch all \
        -rowstretch all 

    #make column 0 narrower
    $t width 0 15

    # make column 1 wider
    $t width 1 30

    $t configure -usecommand 1 -command {
        if {%r>0 && %c>0} {
            global currGodley
            minsky.godleyItem.get $currGodley
            set row [expr %r-1]
            set col [expr %c-1]
            if [minsky.godleyItem.table.doubleEntryCompliant] {
                # allow for asset class row
                incr row -1
                if {$row==-1} return "";
                # TODO: compute row sum here
                if {$col==[minsky.godleyItem.table.cols]} {
                    if {$row>0} {
                        return [minsky.godleyItem.table.rowSum $row]
                    } else {
                        # don't sum column labels
                        return "Row Sum"
                    }
                }
            }
            if {%i && $currGodley>=0} {
                minsky.godleyItem.table.setCell $row $col "%s"
                minsky.godleyItem.set $currGodley
                updateGodley
            } else {
                set s [minsky.godleyItem.table.getCell $row $col]
                if [string length $s] {
                    return $s
                } else {
                    return " "
                }
            }
        }
    }

    # required to make this behave on Windows.            
    $t tag configure active -foreground black    

    frame .godley.topbar -width 400 
    label .godley.topbar.label -text "Godley Table"

    scrollbar .godley.sy -command [list $t yview]
    scrollbar .godley.sx -command [list $t xview] -orient horizontal

    checkbutton .godley.doubleEntryMode -text "Double Entry" -variable DEmode \
        -command {
            godleyItem.get $currGodley
            godleyItem.table.setDEmode $DEmode
            godleyItem.set
            updateGodley
        }
    # default checkbox mode is deselected - select if model variable is set
    if [godleyItem.table.doubleEntryCompliant] {
        .godley.doubleEntryMode select
    }
    pack .godley.topbar 
    pack .godley.sy -side right -fill y
    pack .godley.table -fill both 
    pack .godley.sx -fill x

#    bind .godley <Destroy> "rename $t {}"
}




proc openGodley {id} {
    if {![winfo exists .godley]} {createGodleyWindow}

    global currGodley
    set currGodley $id
    wm deiconify .godley
    raise .godley .
    updateGodley
}

proc addRow {r} {
    global currGodley
    godleyItem.get $currGodley
    godleyItem.table.insertRow $r
    godleyItem.set $currGodley
    # if we don't remove activation, sometimes the cell content is not correctly updated
    .godley.table activate 0,0
    updateGodley
}

proc delRow {r} {
    global currGodley
    godleyItem.get $currGodley
    godleyItem.table.deleteRow $r
    godleyItem.set $currGodley
    .godley.table activate 0,0
    updateGodley
}

proc addCol {c} {
    global currGodley
    godleyItem.get $currGodley
    godleyItem.table.insertCol $c
    godleyItem.set $currGodley
    .godley.table activate 0,0
    updateGodley
}

proc delCol {c} {
    global currGodley   
    godleyItem.get $currGodley
    godleyItem.table.deleteCol $c
    godleyItem.set $currGodley
    .godley.table activate 0,0
    updateGodley
}
    

proc updateGodley {} {
    global currGodley   
    godleyItem.get $currGodley
    set nrows [expr [godleyItem.table.rows]+1]
    set ncols [expr [godleyItem.table.cols]+1]
    if [godleyItem.table.doubleEntryCompliant] {
        incr nrows
        incr ncols
    }

    .godley.table configure -rows $nrows -cols $ncols

    # delete row/col buttons
    foreach c [info commands .godley.???Buttons{*}] {destroy $c}
    # delete asset class dropdowns
    foreach c [info commands .godley.assetClass{*}] {destroy $c}

    # put the double entry book keeping mode button in top left corner
    .godley.table window configure 0,0 -window .godley.doubleEntryMode

    for {set r 1} {$r<[expr [godleyItem.table.rows]+1]} {incr r} {
        set ro [expr $r+[godleyItem.table.doubleEntryCompliant]]
        frame .godley.rowButtons{$r}
        button .godley.rowButtons{$r}.add -foreground green -text "+" -command "addRow $r"
        pack .godley.rowButtons{$r}.add -side left
        if {$r>1} {
            button .godley.rowButtons{$r}.del -foreground red -text "-" -command "delRow $r"
            pack .godley.rowButtons{$r}.del -side left
        }
        .godley.table window configure $ro,0 -window .godley.rowButtons{$r}
    }
    for {set c 1} {$c<[expr [godleyItem.table.cols]+1]} {incr c} {
        frame .godley.colButtons{$c}
        button .godley.colButtons{$c}.add -foreground green -text "+" -command "addCol $c"
        pack .godley.colButtons{$c}.add  -side left
        if {$c>1} {
            button .godley.colButtons{$c}.del -foreground red -text "-" -command "delCol $c"
            pack .godley.colButtons{$c}.del -side left
        }

        .godley.table window configure 0,$c -window .godley.colButtons{$c}

        if {$c>1 && [godleyItem.table.doubleEntryCompliant]} {
            # C++ table column offset by one wrt TkTable columns
            set col [expr $c-1]
            menubutton .godley.assetClass{$c} -menu .godley.assetClass{$c}.menu\
                -text [godleyItem.table.assetClass $col] -relief raised
            menu .godley.assetClass{$c}.menu
            foreach assetClass [assetClasses] {
                .godley.assetClass{$c}.menu add command -label $assetClass \
                    -command "
                      godleyItem.table.assetClass $col $assetClass
                      godleyItem.set
                      updateGodley
                    "
            }
           .godley.table window configure 1,$c -window .godley.assetClass{$c}
        }
    }

    godleyItem.update
    godleyItem.set $currGodley
    updateGodleyItem $currGodley
    update
}


