#  @copyright Steve Keen 2012
#  @author Russell Standish
#  This file is part of Minsky.
#
#  Minsky is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Minsky is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Minsky.  If not, see <http://www.gnu.org/licenses/>.
#

.menubar.ops.menu add command -label Plot -command "newPlot"

set nextPlotImage 0

proc newPlotItem {image x y} {
    global nextPlotImage
    if  {[lsearch -exact [image names] $image]!=-1} {
        image delete $image
    }
    image create photo $image -width 150 -height 150
    
    set id [.wiring.canvas create plot $x $y -image $image -scale 1 -rotation 0\
                -tags "plots plot#$image"]
    .wiring.canvas bind $id <B1-Motion> "movePlot $image %x %y"
    .wiring.canvas bind $id <B1-ButtonRelease> "movePlot $image %x %y"
    .wiring.canvas bind $id <Double-Button-1> "plotDoubleClick $image"
    .wiring.canvas bind $id <Button-3> "contextMenu $id %x %y"
}

proc newPlot {} {
    # place this at the mouse if in canvas, otherwise at 0 0
    set xy [placeObject]

    set image [plots.nextPlotID]
    set id [eval newPlotItem $image $xy]

    bind .wiring.canvas <Enter> "movePlot $image %x %y"
    bind .wiring.canvas <Motion> "movePlot $image %x %y"
    bind .wiring.canvas <Button-1> \
        "bind .wiring.canvas <Motion> {}; bind .wiring.canvas <Enter> {}"

    eval plots.moveTo $image $xy
#    updateCanvas
    return $id
}

set updatePlotPositionSubmitted 0

proc updatePlotPosition {image} {
    global updatePlotPositionSubmitted
    plot.get $image
    .wiring.canvas coords plot#$image [plot.x] [plot.y]
    foreach port [plot.ports] {
        adjustWire $port
    }
    set updatePlotPositionSubmitted 0   
}

proc movePlot {image x y} {
    plots.moveTo  $image $x $y
    # for some reason, we should queue up screen updates to happen once idle, to prevent Tk from getting overloaded
    global updatePlotPositionSubmitted
    if {!$updatePlotPositionSubmitted} {
        # only submitted if no update already scheduled
        set updatePlotPositionSubmitted 1
        after idle updatePlotPosition $image
    }
}

# double click handling for plot (creates new toplevel plot window)
proc plotDoubleClick {image} {
#    set tags [.wiring.canvas gettags $item]
#    set image [lindex $tags [lsearch -regexp $tags {plot#[[:alnum:]]+}]]

    toplevel .plot$image
    image create photo .plot$image.image -width 500 -height 500
    label .plot$image.label -image .plot$image.image
    pack .plot$image.label

    plots.addImage $image .plot$image.image
    #TODO setting plot image again duplicates an image reference!
#    bind .plot$image <Configure> "
#      puts \"in plots configure %h %w\"
#      .plot$image.image configure -height %h -width %w
#      plot.get $image 
#      plot.image $image
#      plot.redraw
#    "
}
    
proc deletePlot {item image} {
    .wiring.canvas delete $item
    minsky.deletePlot $image
    updateCanvas
}
    
