#  @copyright Steve Keen 2012
#  @author Russell Standish
#  This file is part of Minsky.
#
#  Minsky is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Minsky is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Minsky.  If not, see <http://www.gnu.org/licenses/>.
#

# Wiring canvas

proc createWire {coords} {
    .wiring.canvas create line $coords -tag wires -arrow last -smooth bezier
}

#toplevel .wiring 
frame .wiring 
pack .wiring -fill both -expand 1
#menu .wiring.menubar -type menubar
frame .wiring.menubar 

#.wiring.menubar add cascade -menu .wiring.menubar.ops -label Operations
#.wiring configure -menu .wiring.menubar

# move mode initially
set interactionMode 2 
radiobutton .wiring.menubar.wiringmode -value 1 -variable interactionMode -command setInteractionMode -text wire
radiobutton .wiring.menubar.movemode -value 2 -variable interactionMode -command setInteractionMode -text move
radiobutton .wiring.menubar.panmode -value 3 -variable interactionMode -command setInteractionMode -text pan
radiobutton .wiring.menubar.lassomode -value 4 -variable interactionMode -command setInteractionMode -text lasso

image create photo godleyImg -file $minskyHome/icons/bank.gif
button .wiring.menubar.godley -image godleyImg -height 24 -width 37 \
    -command {newGodleyItem [addGodleyTable 10 10]}
tooltip .wiring.menubar.godley "Godley table"

image create photo varImg -file $minskyHome/icons/var.gif
button .wiring.menubar.var -image varImg -height 24 -width 37 \
    -command addVariable
tooltip .wiring.menubar.var "variable"

image create photo constImg -file $minskyHome/icons/const.gif
button .wiring.menubar.const -height 24 -width 37 -image constImg -command {
    addOperation constant}
tooltip .wiring.menubar.const "constant"

image create photo integrateImg -file $minskyHome/icons/integrate.gif
button .wiring.menubar.integrate -image integrateImg -command {
    addOperation integrate}
tooltip .wiring.menubar.integrate integrate

image create photo expImg -file $minskyHome/icons/exp.gif
button .wiring.menubar.exp -image expImg -command {addOperation exp}
tooltip .wiring.menubar.exp exp

image create photo addImg -file $minskyHome/icons/add.gif
button .wiring.menubar.add -image addImg -command {addOperation add}
tooltip .wiring.menubar.add add

image create photo subtractImg -file $minskyHome/icons/subtract.gif
button .wiring.menubar.subtract -image subtractImg -command {
    addOperation subtract}
tooltip .wiring.menubar.subtract subtract

image create photo multiplyImg -file $minskyHome/icons/multiply.gif
button .wiring.menubar.multiply -image multiplyImg -command {
    addOperation multiply}
tooltip .wiring.menubar.multiply multiply

image create photo divideImg -file $minskyHome/icons/divide.gif
button .wiring.menubar.divide -image divideImg -command {addOperation divide}
tooltip .wiring.menubar.divide divide

image create photo plotImg -file $minskyHome/icons/plot.gif
button .wiring.menubar.plot -image plotImg -height 24 -width 37 \
    -command {newPlot}
tooltip .wiring.menubar.plot "Plot"

pack .wiring.menubar.wiringmode .wiring.menubar.movemode .wiring.menubar.panmode .wiring.menubar.lassomode -side left
pack .wiring.menubar.godley .wiring.menubar.var .wiring.menubar.const -side left
pack .wiring.menubar.integrate .wiring.menubar.exp .wiring.menubar.add .wiring.menubar.subtract .wiring.menubar.multiply .wiring.menubar.divide .wiring.menubar.plot -side left
pack .wiring.menubar -fill x


canvas .wiring.canvas -height 600 -width 800 -scrollregion {0 0 10000 10000} \
    -closeenough 2
pack .wiring.canvas -fill both -expand 1
#bind .wiring.canvas <B1-ButtonPress> ".wiring.canvas scan mark %x %y"
bind .wiring.canvas <Control-B1-Motion> "moveCanvas %x %y"
bind .wiring.canvas <B1-ButtonRelease> "set buttonPressed 0"

set buttonPressed 0
proc moveCanvas {x y} {
    global buttonPressed
    if {!$buttonPressed} {
        .wiring.canvas scan mark $x $y
        set buttonPressed 1
    }
    .wiring.canvas scan dragto $x $y 1
}

.menubar.ops.menu add command -label "Godley Table" -command {newGodleyItem [addGodleyTable 10 10]}

.menubar.ops.menu add command -label "Variable" -command "addVariable" 
foreach var [availableOperations] { 
    .menubar.ops.menu add command -label $var -command "addOperation $var"
}

# returns where to place an object: at the mouse if in canvas, otherwise at 0 0
proc placeObject {} {
    set x [winfo pointerx .wiring.canvas]
    set y [winfo pointery .wiring.canvas]
    incr x [expr -[winfo rootx .wiring.canvas]]
    incr y [expr -[winfo rooty .wiring.canvas]]
    return "[.wiring.canvas canvasx $x] [.wiring.canvas canvasy $y]"
}

proc placeNewVar {id} {
    var.get $id
    eval var.moveTo [placeObject]
    var.set
    bind .wiring.canvas <Enter> "moveVar $id %x %y"
    bind .wiring.canvas <Motion> "moveVar $id %x %y"
    bind .wiring.canvas <Button-1> \
        "bind .wiring.canvas <Motion> {}; bind .wiring.canvas <Enter> {}"
}

proc addVariablePostModal {} {
    set name [string trim [.wiring.editConstant.text.value get]]
    set varExists [variables.exists $name]
    set id [newVariable $name]
    if {!$varExists} {
        var.get $id
        value.get $name
        setItem value init {.wiring.editConstant.val.value get}
        setItem var rotation {.wiring.editConstant.rotation.value get}
    }
    closeEditWindow .wiring.editConstant
    placeNewVar $id
}

proc addVariable {} {
    .wiring.editConstant.title configure -text "Create Variable"
    .wiring.editConstant.text.value delete 0 end
    .wiring.editConstant.text.value insert 0 ""
    .wiring.editConstant.val.value delete 0 end
    .wiring.editConstant.val.value insert 0 0
    .wiring.editConstant.buttonBar.ok configure \
        -command "addVariablePostModal"
    wm deiconify .wiring.editConstant
    grab .wiring.editConstant
}

proc addOperation {op} {
    placeNewOp [minsky.addOperation $op]
}

proc placeNewOp {opid} {
    op.get $opid
    eval op.moveTo [placeObject]
    op.set $opid
    drawOperation $opid
    bind .wiring.canvas <Enter> "move op $opid op$opid %x %y"
    bind .wiring.canvas <Motion> "move op $opid op$opid %x %y"
    bind .wiring.canvas <Button-1> \
        "bind .wiring.canvas <Motion> {}; bind .wiring.canvas <Enter> {}"
}

proc drawOperation {id} {
    op.get $id

    if {[lsearch -exact [image name] opImage$id]!=-1} {
        image delete opImage$id
    }
    image create photo opImage$id -width 200 -height 200

    .wiring.canvas delete op$id
    .wiring.canvas create operation [op.x] [op.y] -id $id -image opImage$id -tags "op$id operations" 
#    .wiring.canvas create rectangle [.wiring.canvas bbox op$id] -tags op$id

    setM1Binding op $id op$id
    op.get $id
    .wiring.canvas bind op$id <B2-Motion> \
        "wires::extendConnect [lindex [op.ports] 0] op$id %x %y"
    .wiring.canvas bind op$id <B2-ButtonRelease> \
        "wires::finishConnect op$id %x %y"
    .wiring.canvas bind op$id  <Button-3> "contextMenu op$id %X %Y"
    .wiring.canvas bind op$id  <Double-Button-1> "doubleClick op$id %X %Y"
}

proc updateItemPos {tag item id} {
    global updatePlotPositionSubmitted
    $item.get $id
    eval .wiring.canvas coords $tag [$item.x] [$item.y]
    foreach p [$item.ports]  {
        adjustWire $p
    }
    set updatePlotPositionSubmitted 0   
}    

proc move {item id tag x y} {
    $item.get $id
    $item.moveTo [.wiring.canvas canvasx $x] [.wiring.canvas canvasy $y]
    $item.set $id
    global updatePlotPositionSubmitted
    if {!$updatePlotPositionSubmitted} {
        # only submitted if no update already scheduled
        set updatePlotPositionSubmitted 1
        after idle updateItemPos $tag $item $id
    }
}

proc moveVar {id x y} {
    var.get $id
    set x [.wiring.canvas canvasx $x]
    set y [.wiring.canvas canvasy $y]
   .wiring.canvas coords var#$id $x $y
    var.moveTo $x $y
    var.set
    adjustWire [var.outPort]
    adjustWire [var.inPort]
    update
}

# create a new canvas item for var id
proc newVar {id} {
    var.get $id
    if {[lsearch -exact [image name] varImage$id]!=-1} {
        image delete varImage$id
    }
    image create photo varImage$id -width 200 -height 50
    
    .wiring.canvas delete var#$id
    set itemId [.wiring.canvas create variable [var.x] [var.y] -image varImage$id -id $id -tags "variables var#$id"]
    # wire drawing. Can only start from an output port
    .wiring.canvas bind var#$id <B2-Motion> \
        "wires::extendConnect [var.outPort] var#$id %x %y"
    .wiring.canvas bind var#$id <B2-ButtonRelease> "wires::finishConnect var#$id %x %y"

    # context menu
    .wiring.canvas bind var#$id <Button-3> "contextMenu $itemId %X %Y"
    .wiring.canvas bind var#$id <Double-Button-1> "doubleClick $itemId %X %Y"
    
}

proc newGodleyItem {id} {
    global minskyHome
    godleyItem.get $id
    if {[lsearch -exact [image name] godleyImage$id]!=-1} {
        image delete godleyImage$id
    }
    image create photo godleyImage$id -width 1000 -height 1000
    .wiring.canvas create godley [godleyItem.x] [godleyItem.y] -image godleyImage$id -id $id -xgl $minskyHome/icons/bank.xgl -tags "godleys godley$id"
#    .wiring.canvas create rectangle [.wiring.canvas bbox godley$id]

    setM1Binding godleyItem $id godley$id
    .wiring.canvas bind godley$id <B2-Motion> \
        "wires::extendConnect \[closestOutPort %x %y \] godley$id %x %y"
    .wiring.canvas bind godley$id <B2-ButtonRelease> \
        "wires::finishConnect godley$id %x %y"
    .wiring.canvas bind godley$id  <Button-3> "rightMouseGodley $id %x %y %X %Y"
    .wiring.canvas bind godley$id  <Double-Button-1> "doubleMouseGodley $id %x %y"
}

proc rightMouseGodley {id x y X Y} {
    godleyItem.get $id
    set var [godleyItem.select [.wiring.canvas canvasx $x] [.wiring.canvas canvasy $y]]
    if {$var==-1} {
        contextMenu godley$id $X $Y
    } else {
        .wiring.context delete 0 end
        .wiring.context add command -label "Edit" -command "editItem $var var"
        var.get $var
        .wiring.context add command -label "Copy" -command "
           copyVar $var
           var.rotation 0
           var.set
        "
        .wiring.context post $X $Y

    }
}

proc doubleMouseGodley {id x y} {
    godleyItem.get $id
    set var [godleyItem.select [.wiring.canvas canvasx $x] [.wiring.canvas canvasy $y]]
    if {$var==-1} {
        openGodley $id
    } else {
        editItem $var var
    }
}

proc updateGodleyItem {id} {
    .wiring.canvas delete godley$id
    newGodleyItem $id
}

proc newWire {wire wireid} {
    .wiring.canvas addtag wire$wireid withtag $wire
    .wiring.canvas bind $wire <Enter> "decorateWire $wireid"
    .wiring.canvas bind $wire <Button-3> "contextMenu $wire %X %Y"
}

proc updateCoords {wire handle pos x y} {
#    assert {[llength [.wiring.canvas find withtag wire$wire]]==1}
    set coords [.wiring.canvas coords wire$wire]
    set x0 [lindex $coords $pos]
    set y0 [lindex $coords [expr $pos+1]]
    .wiring.canvas move $handle [expr $x-$x0] [expr $y-$y0]
    lset coords $pos $x
    lset coords [expr $pos+1] $y
    .wiring.canvas coords wire$wire $coords
    wireCoords $wire $coords
}

proc insertCoords {wire handle pos x y} {
    global handles
#    assert {[llength [.wiring.canvas find withtag wire$wire]]==1}
    if {![info exists handles($handle)]} {
        set handles($handle) 1
        # add current handle coordinates to the wire shape
        set coords [.wiring.canvas coords wire$wire]
        set coords [linsert $coords $pos $x $y]
        .wiring.canvas coords wire$wire $coords
    }
    updateCoords $wire $handle $pos $x $y
}

proc createHandle {x y} {
    return [
        .wiring.canvas create oval \
        [expr $x-3] [expr $y-3] [expr $x+3] [expr $y+3] \
        -fill blue  -tag handles
    ]
}

proc deleteHandle {wire handle pos} {
    .wiring.canvas delete $handle
    set coords [lreplace [.wiring.canvas coords $wire] $pos [expr $pos+1]]
    .wiring.canvas coords $wire $coords        
    wireCoords $wire $coords
    decorateWire $wire
}
    

proc decorateWire {wire} {
#    global TCLwireid
#    set wireid $TCLwireid($wire)
    .wiring.canvas delete handles
    set coords [.wiring.canvas coords wire$wire]
    for {set i 0} {$i<[llength $coords]-2} {incr i 2} {
        if {$i>0} {
            set h [
                   createHandle \
                       [lindex $coords $i] [lindex $coords [expr $i+1]]
                  ]
            .wiring.canvas bind $h <B1-Motion> \
                "updateCoords $wire $h $i %x %y"
            .wiring.canvas bind $h <Double-Button-1> \
                "deleteHandle $wire $h $i"
            }
        # create a handle in between
        set h [
            createHandle [
                expr ([lindex $coords $i]+[lindex $coords [expr $i+2]])/2
            ] [
                expr ([lindex $coords [expr $i+1]]+\
                    [lindex $coords [expr $i+3]])/2]
        ]
        .wiring.canvas bind $h <B1-Motion> \
            "insertCoords $wire $h [expr $i+2] %x %y" 
        .wiring.canvas bind $h <Double-Button-1> \
            "deleteHandle $wire $h [expr $i+2]"
    }
}

namespace eval wires {
    proc extendConnect {portId id x y} {
        set x [.wiring.canvas canvasx $x]
        set y [.wiring.canvas canvasy $y]
        namespace children
        if {![namespace exists $id]} {
            namespace eval $id {
                variable wire
                variable fromPort
                variable x0
                variable y0 
               
                proc extendConnect {x y} {
                    variable wire
                    variable x0
                    variable y0 
                    .wiring.canvas coords $wire $x0 $y0 $x $y
                }
                
                proc finishConnect {x y} {
                    set x [.wiring.canvas canvasx $x]
                    set y [.wiring.canvas canvasy $y]
                    variable wire
                    variable fromPort
                    variable x0
                    variable y0 
                    set portId [closestInPort $x $y]
                    if {$portId>=0} {
#                        set port [ports.@elem $portId]
                        eval .wiring.canvas coords $wire $x0 $y0 [portCoords $portId]
                        set wireid [addWire $fromPort $portId \
                                        [.wiring.canvas coords $wire]]
                        if {$wireid == -1} {
                            # wire is invalid
                            .wiring.canvas delete $wire
                        } else {
                            newWire $wire $wireid
                        }
                    } else {
                        .wiring.canvas delete $wire
                    }
                    namespace delete [namespace current]
                }

            }

            set [set id]::wire [createWire "$x $y $x $y"]
#            set port [ports.@elem $portId]
            set [set id]::fromPort $portId
            set portCoords [portCoords $portId]
            set [set id]::x0 [lindex $portCoords 0]
            set [set id]::y0 [lindex $portCoords 1]
        }

        [set id]::extendConnect $x $y
    }

    proc finishConnect {id x y} {[set id]::finishConnect $x $y}
}

# adjust the begin or end of line when a port moves
proc adjustWire {portId} {
    foreach w [wiresAttachedToPort $portId] {
        .wiring.canvas delete handles
        wire.get $w
        if [wire.visible] {
            # ensure wire exists on canvas
            if {[.wiring.canvas type wire$w]==""} {
                newWire [createWire [wire.coords]] $w
            }
            eval .wiring.canvas coords wire$w [wire.coords]
        }
   }
}

proc straightenWire {id} {
    wire.get $id
    port.get [wire.from]
    set fx [port.x]
    set fy [port.y]
    port.get [wire.to]
    wire.coords "$fx $fy [port.x] [port.y]"
    wire.set
    eval .wiring.canvas coords wire$id [wire.coords]
}

proc setM1Binding {item id tag} {
    global interactionMode
    switch $interactionMode {
        1 {
            $item.get $id
            # wiring mode
            .wiring.canvas bind $tag <B1-Motion> \
                "wires::extendConnect \[closestOutPort %x %y\] $tag %x %y"
            .wiring.canvas bind $tag <B1-ButtonRelease>  \
                "wires::finishConnect $tag %x %y"
        }
        2 { 
            # move mode
            .wiring.canvas bind $tag <B1-Motion> "move $item $id $tag %x %y"
            .wiring.canvas bind $tag <B1-ButtonRelease> "move $item $id $tag %x %y"
        }
        default { 
            # pan mode
            .wiring.canvas bind $tag <B1-Motion> ""
            .wiring.canvas bind $tag <B1-ButtonRelease> ""
        }
    }
}

proc setInteractionMode {} {
    global interactionMode

    bind .wiring.canvas <B1-Motion> ""
    bind .wiring.canvas <B1-ButtonRelease> ""
    
    switch $interactionMode {
        3 {
            bind .wiring.canvas <B1-Motion> "moveCanvas %x %y"
        } 
        4 {
            bind .wiring.canvas <B1-Motion> "lasso %x %y"
            bind .wiring.canvas <B1-ButtonRelease> "lassoEnd %x %y"
        }
    }


    foreach var [variables.#keys] {
        var.get $var
        switch $interactionMode {
            1 { 
                # wiring mode
                .wiring.canvas bind var#$var <B1-Motion> "wires::extendConnect [var.outPort] var#$var %x %y"
                .wiring.canvas bind var#$var <B1-ButtonRelease> "wires::finishConnect var#$var %x %y"
            }
            2 { 
                # move mode
                .wiring.canvas bind var#$var <B1-Motion> "moveVar $var %x %y"
                .wiring.canvas bind var#$var <B1-ButtonRelease> ""
            }
            default { 
                # pan/lasso mode
                .wiring.canvas bind var#$var <B1-Motion> ""
                .wiring.canvas bind var#$var <B1-ButtonRelease> ""
            }
        } 
    }

    foreach op [operations.#keys] {setM1Binding op $op op$op}
    foreach id [godleyItems.#keys] {setM1Binding godleyItem $id godley$id}
    foreach id [groupItems.#keys] {setM1Binding groupItem $id group$id}
}

proc updateCanvas {} {
    global fname showPorts
#    wm title .wiring "Wiring diagram: $fname (Prototype)"
#    .wiring.canvas delete variables operations wires handles plots
    .wiring.canvas delete all
    foreach var [variables.visibleVariables] {
        newVar $var

    }

    setInteractionMode

    # groups need to be done first, as they adjust port positions (hence wires)
    foreach g [groupItems.#keys] {
        newGroupItem $g
    }

    # TODO add operations
    foreach o [operations.visibleOperations] {
        drawOperation $o
    }

    # add wires to canvas
    foreach w [visibleWires] {
#        set wire [wires.@elem $w]
        set id [createWire [wireCoords $w]]
        newWire $id $w 
    }

    foreach im [plots.plots.#keys] {
        newPlotItem $im [plots.X $im] [plots.Y $im]
    }

    foreach g [godleyItems.#keys] {
        newGodleyItem $g
    }


# the following loop helps debug port placement
    if {$showPorts} {
        foreach port [ports.#keys] {
            port.get $port
            .wiring.canvas create oval \
                [expr [port.x]-2] [expr [port.y]-2] [expr [port.x]+2] [expr [port.y]+2] \
                -fill {} -outline blue 
        }
    }
  
# adjust wire coordinates to where the ports actually are
#    foreach port [ports.#keys] {
#        adjustWire $port
#    }

}

menu .wiring.context -tearoff 0

proc doubleClick {item x y} {
    # find out what type of item we're referring to
    set tags [.wiring.canvas gettags $item]
    switch -regexp $tags {
        "variables" {
            set tag [lindex $tags [lsearch -regexp $tags {var#[[:alnum:]]+}]]
            set id [string range $tag 4 end]
            editItem $id $tag
        }
        "operations" {
            set tag [lindex $tags [lsearch -regexp $tags {op[0-9]+}]]
            set id [string range $tag 2 end]
            editItem $id $tag
        }
    }
}

proc toggleCoupled {id} {
    op.get $id
    op.toggleCoupled
    op.set
    updateCanvas
}

# context menu
proc contextMenu {item x y} {
    # find out what type of item we're referring to
    set tags [.wiring.canvas gettags $item]
    switch -regexp $tags {
        "variables" {
            set tag [lindex $tags [lsearch -regexp $tags {var#[[:alnum:]]+}]]
            set id [string range $tag 4 end]
            .wiring.context delete 0 end
            .wiring.context add command -label "Delete" -command "deleteItem $id $tag"
            .wiring.context add command -label "Edit" -command "editItem $id $tag"
            .wiring.context add command -label "Copy" -command "copyVar $id"
            .wiring.context add command -label "Flip" -command "rotateVar $id 180"
        }
        "operations" {
            set tag [lindex $tags [lsearch -regexp $tags {op[0-9]+}]]
            set id [string range $tag 2 end]
            op.get $id
            .wiring.context delete 0 end
            .wiring.context add command -label "Delete" -command "deleteItem $id $tag"
            .wiring.context add command -label "Edit" -command "editItem $id $tag"             
            if {[op.name]=="integrate"} {
                .wiring.context add command -label "Copy Var" -command "copyVar [op.intVarID]"
            }
            .wiring.context add command -label "Copy" -command "copyOp $id"
            .wiring.context add command -label "Flip" -command "rotateOp $id 180"
            op.get $id
            if {[op.name]=="integrate"} {
                .wiring.context add command -label "Toggle var binding" -command "toggleCoupled $id"
            }
        }
        "wires" {
            set tag [lindex $tags [lsearch -regexp $tags {wire[0-9]+}]]
            set id [string range $tag 4 end]
            .wiring.context delete 0 end
            .wiring.context add command -label "Straighten" -command "straightenWire $id"
            .wiring.context add command -label "Delete" -command "deleteItem $id $tag"
        }
        "plots" {
            set tag [lindex $tags [lsearch -regexp $tags {plot#.+}]]
            set id [string range $tag 5 end]
            .wiring.context delete 0 end
            .wiring.context add command -label "Expand" -command "plotDoubleClick $id"
            .wiring.context add command -label "Delete" -command "deletePlot $item $id"
        }
        "godleys" {
            set tag [lindex $tags [lsearch -regexp $tags {godley[0-9]+}]]
            set id [string range $tag 6 end]
            .wiring.context delete 0 end
            .wiring.context add command -label "Open Godley Table" -command "openGodley $id"
            .wiring.context add command -label "Delete Godley Table" -command "deleteItem $id $tag"
        }
        "group" {
            set tag [lindex $tags [lsearch -regexp $tags {group[0-9]+}]]
            set id [string range $tag 5 end]
            groupContext $id $x $y
        }
    }
#    .wiring.context post $x $y
    tk_popup .wiring.context $x $y
}

proc deleteItem {id tag} {
    .wiring.canvas delete $tag
    switch -regexp $tag {
        "^op" {
            deleteOperation $id
            updateCanvas
        }
        "^wire" {
            .wiring.canvas delete handles
            deleteWire $id
        }
        "^var" {
            deleteVariable $id
            updateCanvas
        }
        "^godley" {
            deleteGodleyTable $id
            updateCanvas
        }
    }
}

proc copyVar {id} {
    set newId [copyVariable $id]
    newVar $newId
    placeNewVar $newId
}
proc copyOp  {id} {placeNewOp [copyOperation $id]}

proc rotateOp {id angle} {
    op.get $id
    op.rotation [expr [op.rotation]+$angle]
    op.set
    drawOperation $id
    foreach p [op.ports] {
        adjustWire $p
    }
}

proc rotateVar {id angle} {
    var.get $id
    var.rotation [expr [var.rotation]+$angle]
    var.set
    newVar $id
    adjustWire [var.outPort]
    adjustWire [var.inPort]
}

toplevel .wiring.editVar 
wm title .wiring.editVar "Edit Variable"
wm withdraw .wiring.editVar
wm transient .wiring.editVar .wiring

label .wiring.editVar.name
pack .wiring.editVar.name

frame .wiring.editVar.init
label .wiring.editVar.init.label -text "Initial Value"
entry  .wiring.editVar.init.val -width 20
pack .wiring.editVar.init.label .wiring.editVar.init.val -side left

frame .wiring.editVar.rot
label .wiring.editVar.rot.label -text "     Rotation"
entry  .wiring.editVar.rot.val -width 20
pack .wiring.editVar.rot.label .wiring.editVar.rot.val -side left

pack .wiring.editVar.init .wiring.editVar.rot

frame .wiring.editVar.buttonBar
button .wiring.editVar.buttonBar.ok -text OK
button .wiring.editVar.buttonBar.cancel -text Cancel -command {
    closeEditWindow .wiring.editVar}
pack .wiring.editVar.buttonBar.ok .wiring.editVar.buttonBar.cancel -side left
pack .wiring.editVar.buttonBar -side bottom
bind .wiring.editVar <Key-Return> {invokeOKorCancel .wiring.editVar.buttonBar}

toplevel .wiring.editConstant
wm title .wiring.editConstant "Edit Constant"
wm withdraw .wiring.editConstant
wm transient .wiring.editConstant .wiring

frame .wiring.editConstant.buttonBar
label .wiring.editConstant.title
pack .wiring.editConstant.title
button .wiring.editConstant.buttonBar.ok -text OK
button .wiring.editConstant.buttonBar.cancel -text Cancel -command {
    closeEditWindow .wiring.editConstant}
pack .wiring.editConstant.buttonBar.ok .wiring.editConstant.buttonBar.cancel -side left
pack .wiring.editConstant.buttonBar -side bottom

frame .wiring.editConstant.text
label .wiring.editConstant.text.label -text "Name"
entry  .wiring.editConstant.text.value -width 20
pack .wiring.editConstant.text.label .wiring.editConstant.text.value -side left
frame .wiring.editConstant.val
label .wiring.editConstant.val.label -text "Value"
entry  .wiring.editConstant.val.value -width 20
pack .wiring.editConstant.val.label .wiring.editConstant.val.value -side left
frame .wiring.editConstant.rotation
label .wiring.editConstant.rotation.label -text "Rotation"
entry  .wiring.editConstant.rotation.value -width 20
pack .wiring.editConstant.rotation.label .wiring.editConstant.rotation.value -side left
pack .wiring.editConstant.text .wiring.editConstant.val .wiring.editConstant.rotation

bind .wiring.editConstant <Key-Return> {invokeOKorCancel .wiring.editConstant.buttonBar}


toplevel .wiring.editOperation
wm title .wiring.editOperation "Edit Constant"
wm withdraw .wiring.editOperation
wm transient .wiring.editOperation .wiring

frame .wiring.editOperation.buttonBar
label .wiring.editOperation.title
pack .wiring.editOperation.title
button .wiring.editOperation.buttonBar.ok -text OK
button .wiring.editOperation.buttonBar.cancel -text Cancel -command {
    closeEditWindow .wiring.editOperation}
pack .wiring.editOperation.buttonBar.ok .wiring.editOperation.buttonBar.cancel -side left
pack .wiring.editOperation.buttonBar -side bottom

bind .wiring.editOperation <Key-Return> {invokeOKorCancel .wiring.editOperation.buttonBar}

frame .wiring.editOperation.rotation
label .wiring.editOperation.rotation.label -text "Rotation"
entry  .wiring.editOperation.rotation.value -width 20
pack .wiring.editOperation.rotation.label .wiring.editOperation.rotation.value -side left
pack .wiring.editOperation.rotation

# set attribute, and commit to original item
proc setItem {modelCmd attr dialogCmd} {
    $modelCmd.$attr [string trim [eval $dialogCmd]]
    $modelCmd.set
}

proc setInitVal {var dialogCmd} {
    value.get $var
    value.init [eval $dialogCmd]
    value.set $var
}

proc closeEditWindow {window} {
    grab release $window
    wm withdraw $window
    updateCanvas
}

proc setConstantValue {} {
    setItem op value {.wiring.editConstant.val.value get}
}

proc setIntegralIValue {} {
    value.get [op.description]
    setItem value init {.wiring.editConstant.val.value get}
}

proc editItem {id tag} {
    switch -regexp $tag {
        "^var" {
            var.get $id
            wm title .wiring.editVar "Edit [var.name]"
            .wiring.editVar.init.val delete 0 end
            value.get [var.name]
            .wiring.editVar.init.val insert 0 [value.init]
            if {[value.godleyOverridden] || [variables.inputWired [var.name]]} {
                .wiring.editVar.init.val configure -state disabled  -foreground gray
            } else {
                .wiring.editVar.init.val configure -state normal  -foreground black
             }
           .wiring.editVar.rot.val delete 0 end
           .wiring.editVar.rot.val insert 0 [var.rotation]
            .wiring.editVar.buttonBar.ok configure \
                -command {
                    setItem value init {.wiring.editVar.init.val get}
                    setItem var rotation  {.wiring.editVar.rot.val get}
                    closeEditWindow .wiring.editVar
                }
            .wiring.editVar.name configure -text "[var.name]: Value=[value.value]"
            wm deiconify .wiring.editVar
            grab .wiring.editVar
        }
        "^op" {
            op.get $id

            if {[op.name]=="constant" || [op.name]=="integrate"} {
                .wiring.editConstant.val.value delete 0 end
                switch [op.name] {
                    constant {
                        .wiring.editConstant.val.label configure -text "Value"
                        .wiring.editConstant.val.value insert 0 [op.value]
                        set setValue setConstantValue
                    }
                    integrate {
                        .wiring.editConstant.val.label configure -text "Initial Value"
                        value.get [op.description]
                        .wiring.editConstant.val.value insert 0 [value.init]
                        set setValue setIntegralIValue
                    }
                }
                .wiring.editConstant.title configure -text [op.name]
                .wiring.editConstant.text.value delete 0 end
                .wiring.editConstant.text.value insert 0 [op.description]
                .wiring.editConstant.rotation.value delete 0 end
                .wiring.editConstant.rotation.value insert 0 [op.rotation]
                # value needs to be regotten, as var name may have changed
                .wiring.editConstant.buttonBar.ok configure \
                    -command "
                        setItem op description {.wiring.editConstant.text.value get}
                        $setValue
                        setItem op rotation {.wiring.editConstant.rotation.value get}
                        closeEditWindow .wiring.editConstant
                    "
                wm deiconify .wiring.editConstant
                grab .wiring.editConstant

            } else {
                .wiring.editOperation.title configure -text [op.name]
                .wiring.editOperation.rotation.value delete 0 end
                .wiring.editOperation.rotation.value insert 0 [op.rotation]
                .wiring.editOperation.buttonBar.ok configure \
                    -command {
                        setItem op rotation {.wiring.editOperation.rotation.value get}
                        closeEditWindow .wiring.editOperation
                    }
                wm deiconify .wiring.editOperation
                grab .wiring.editOperation
            }
        }
        "^group" {groupEdit $id}
    }
}
